package cz.muni.pa165.soa.endpoints;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.ws.server.endpoint.annotation.Endpoint;
import org.springframework.ws.server.endpoint.annotation.PayloadRoot;
import org.springframework.ws.server.endpoint.annotation.RequestPayload;
import org.springframework.ws.server.endpoint.annotation.ResponsePayload;

import cz.muni.pa165.soa.GetBookRequest;
import cz.muni.pa165.soa.GetBookRequestByAuthor;
import cz.muni.pa165.soa.GetBookResponse;
import cz.muni.pa165.soa.repositories.BookRepository;

/**
 * We define an endpoint by means of the Endpoint annotation
 * an endpoint can have several handler methods for different uris/operations
 * 
 */
@Endpoint
public class BookEndpoint {
    	private static final String NAMESPACE_URI = "http://muni.cz/pa165/soa";

	private final BookRepository bookRepository;

        /**
         * We autowire the BookRepository service 
         *  
         */
	@Autowired
	public BookEndpoint(BookRepository bookRepository) {
		this.bookRepository = bookRepository;
	}

        /**
         * 
         *  Payload root defined an handler methods, by using the general base namespace uri
         *  and the specific part for this handler
         *  we mark to return the ResponsePayload and we annotate the variable for the 
         *  RequestPayload
         */
	@PayloadRoot(namespace = NAMESPACE_URI, localPart = "getBookRequest")
	@ResponsePayload
	public GetBookResponse getBook(@RequestPayload GetBookRequest request) {
		
            GetBookResponse response = new GetBookResponse();
            response.setBook(bookRepository.getBookByTitle(request.getTitle()));
            return response;
            
	}
        
         /**
         * 
         *  Another handler method for processing requests for books 
         *  based on authors
         */
        @PayloadRoot(namespace = NAMESPACE_URI, localPart = "getBookRequestByAuthor")
	@ResponsePayload
	public GetBookResponse getBookByAuthor(@RequestPayload GetBookRequestByAuthor request) {
		
            GetBookResponse response = new GetBookResponse();
            response.setBook(bookRepository.getBookByAuthorNameAndSurname(request.getAuthor()));
            return response;
            
	}
}